@@ -263,7 +263,8 @@ enum
   PROP_IS_REGISTERED,
   PROP_IS_REMOTE,
   PROP_INACTIVITY_TIMEOUT,
-  PROP_ACTION_GROUP
+  PROP_ACTION_GROUP,
+  PROP_IS_BUSY
 };
 
 enum
@@ -1180,6 +1181,10 @@ g_application_get_property (GObject    *object,
                         g_application_get_inactivity_timeout (application));
       break;
 
+    case PROP_IS_BUSY:
+      g_value_set_boolean (value, g_application_get_is_busy (application));
+      break;
+
     default:
       g_assert_not_reached ();
     }
@@ -1213,9 +1218,9 @@ g_application_finalize (GObject *object)
 {
   GApplication *application = G_APPLICATION (object);
 
-  g_slist_free_full (application->priv->option_groups, (GDestroyNotify) g_option_group_free);
+  g_slist_free_full (application->priv->option_groups, (GDestroyNotify) g_option_group_unref);
   if (application->priv->main_options)
-    g_option_group_free (application->priv->main_options);
+    g_option_group_unref (application->priv->main_options);
   if (application->priv->packed_options)
     {
       g_slist_free_full (application->priv->option_strings, g_free);
@@ -1343,6 +1348,20 @@ g_application_class_init (GApplicationClass *class)
                          G_TYPE_ACTION_GROUP,
                          G_PARAM_DEPRECATED | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GApplication:is-busy:
+   *
+   * Whether the application is currently marked as busy through
+   * g_application_mark_busy() or g_application_bind_busy_property().
+   *
+   * Since: 2.44
+   */
+  g_object_class_install_property (object_class, PROP_IS_BUSY,
+    g_param_spec_boolean ("is-busy",
+                          P_("Is busy"),
+                          P_("If this application is currently marked busy"),
+                          FALSE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GApplication::startup:
    * @application: the application
@@ -2305,6 +2324,10 @@ g_application_run (GApplication  *application,
 
   g_settings_sync ();
 
+  if (!application->priv->must_quit_now)
+    while (g_main_context_iteration (NULL, FALSE))
+      ;
+
   return status;
 }
 
@@ -2544,7 +2567,10 @@ g_application_mark_busy (GApplication *application)
   application->priv->busy_count++;
 
   if (!was_busy)
-    g_application_impl_set_busy_state (application->priv->impl, TRUE);
+    {
+      g_application_impl_set_busy_state (application->priv->impl, TRUE);
+      g_object_notify (G_OBJECT (application), "is-busy");
+    }
 }
 
 /**
@@ -2570,7 +2596,29 @@ g_application_unmark_busy (GApplication *application)
   application->priv->busy_count--;
 
   if (application->priv->busy_count == 0)
-    g_application_impl_set_busy_state (application->priv->impl, FALSE);
+    {
+      g_application_impl_set_busy_state (application->priv->impl, FALSE);
+      g_object_notify (G_OBJECT (application), "is-busy");
+    }
+}
+
+/**
+ * g_application_get_is_busy:
+ * @application: a #GApplication
+ *
+ * Gets the application's current busy state, as set through
+ * g_application_mark_busy() or g_application_bind_busy_property().
+ *
+ * Returns: %TRUE if @application is currenty marked as busy
+ *
+ * Since: 2.44
+ */
+gboolean
+g_application_get_is_busy (GApplication *application)
+{
+  g_return_val_if_fail (G_IS_APPLICATION (application), FALSE);
+
+  return application->priv->busy_count > 0;
 }
 
 /* Notifications {{{1 */
@@ -2710,16 +2758,12 @@ g_application_notify_busy_binding (GObject    *object,
 /**
  * g_application_bind_busy_property:
  * @application: a #GApplication
- * @object: a #GObject
- * @property: (allow-none): the name of a boolean property of @object
+ * @object: (type GObject.Object): a #GObject
+ * @property: the name of a boolean property of @object
  *
  * Marks @application as busy (see g_application_mark_busy()) while
  * @property on @object is %TRUE.
  *
- * Multiple such bindings can exist, but only one property can be bound
- * per object. Calling this function again for the same object replaces
- * a previous binding. If @property is %NULL, the binding is destroyed.
- *
  * The binding holds a reference to @application while it is active, but
  * not to @object. Instead, the binding is destroyed when @object is
  * finalized.
@@ -2731,45 +2775,78 @@ g_application_bind_busy_property (GApplication *application,
                                   gpointer      object,
                                   const gchar  *property)
 {
-  GClosure *closure = NULL;
   guint notify_id;
+  GQuark property_quark;
+  GParamSpec *pspec;
+  GApplicationBusyBinding *binding;
+  GClosure *closure;
+
+  g_return_if_fail (G_IS_APPLICATION (application));
+  g_return_if_fail (G_IS_OBJECT (object));
+  g_return_if_fail (property != NULL);
+
+  notify_id = g_signal_lookup ("notify", G_TYPE_OBJECT);
+  property_quark = g_quark_from_string (property);
+  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (object), property);
+
+  g_return_if_fail (pspec != NULL && pspec->value_type == G_TYPE_BOOLEAN);
+
+  if (g_signal_handler_find (object, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DETAIL | G_SIGNAL_MATCH_FUNC,
+                             notify_id, property_quark, NULL, g_application_notify_busy_binding, NULL) > 0)
+    {
+      g_critical ("%s: '%s' is already bound to the busy state of the application", G_STRFUNC, property);
+      return;
+    }
+
+  binding = g_slice_new (GApplicationBusyBinding);
+  binding->app = g_object_ref (application);
+  binding->is_busy = FALSE;
+
+  closure = g_cclosure_new (G_CALLBACK (g_application_notify_busy_binding), binding,
+                            g_application_busy_binding_destroy);
+  g_signal_connect_closure_by_id (object, notify_id, property_quark, closure, FALSE);
+
+  /* fetch the initial value */
+  g_application_notify_busy_binding (object, pspec, binding);
+}
+
+/**
+ * g_application_unbind_busy_property:
+ * @application: a #GApplication
+ * @object: (type GObject.Object): a #GObject
+ * @property: the name of a boolean property of @object
+ *
+ * Destroys a binding between @property and the busy state of
+ * @application that was previously created with
+ * g_application_bind_busy_property().
+ *
+ * Since: 2.44
+ */
+void
+g_application_unbind_busy_property (GApplication *application,
+                                    gpointer      object,
+                                    const gchar  *property)
+{
+  guint notify_id;
+  GQuark property_quark;
   gulong handler_id;
 
   g_return_if_fail (G_IS_APPLICATION (application));
   g_return_if_fail (G_IS_OBJECT (object));
+  g_return_if_fail (property != NULL);
 
   notify_id = g_signal_lookup ("notify", G_TYPE_OBJECT);
+  property_quark = g_quark_from_string (property);
 
-  if (property != NULL)
+  handler_id = g_signal_handler_find (object, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DETAIL | G_SIGNAL_MATCH_FUNC,
+                                      notify_id, property_quark, NULL, g_application_notify_busy_binding, NULL);
+  if (handler_id == 0)
     {
-      GParamSpec *pspec;
-      GApplicationBusyBinding *binding;
-
-      pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (object), property);
-      g_return_if_fail (pspec != NULL && pspec->value_type == G_TYPE_BOOLEAN);
-
-      binding = g_slice_new (GApplicationBusyBinding);
-      binding->app = g_object_ref (application);
-      binding->is_busy = FALSE;
-
-      /* fetch the initial value */
-      g_application_notify_busy_binding (object, pspec, binding);
-
-      closure = g_cclosure_new (G_CALLBACK (g_application_notify_busy_binding), binding,
-                                g_application_busy_binding_destroy);
+      g_critical ("%s: '%s' is not bound to the busy state of the application", G_STRFUNC, property);
+      return;
     }
 
-  /* unset a previous binding after fetching the new initial value, so
-   * that we don't switch to FALSE for a brief moment when both the
-   * old and the new property are set to TRUE
-   */
-  handler_id = g_signal_handler_find (object, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC, notify_id,
-                                      0, NULL, g_application_notify_busy_binding, NULL);
-  if (handler_id > 0)
-    g_signal_handler_disconnect (object, handler_id);
-
-  if (closure)
-    g_signal_connect_closure_by_id (object, notify_id, g_quark_from_string (property), closure, FALSE);
+  g_signal_handler_disconnect (object, handler_id);
 }
 
 /* Epilogue {{{1 */
