@@ -753,6 +753,34 @@ test_communicate_utf8 (void)
   g_object_unref (proc);
 }
 
+static void
+test_communicate_nothing (void)
+{
+  GError *error = NULL;
+  GPtrArray *args;
+  GSubprocess *proc;
+  GCancellable *cancellable = NULL;
+  gchar *stdout_buf;
+
+  args = get_test_subprocess_args ("cat", NULL);
+  proc = g_subprocess_newv ((const gchar* const*)args->pdata,
+                            G_SUBPROCESS_FLAGS_STDIN_PIPE
+                            | G_SUBPROCESS_FLAGS_STDOUT_PIPE
+                            | G_SUBPROCESS_FLAGS_STDERR_MERGE,
+                            &error);
+  g_assert_no_error (error);
+  g_ptr_array_free (args, TRUE);
+
+  g_subprocess_communicate_utf8 (proc, "", cancellable, &stdout_buf, NULL, &error);
+  g_assert_no_error (error);
+
+  g_assert_cmpstr (stdout_buf, ==, "");
+
+  g_free (stdout_buf);
+
+  g_object_unref (proc);
+}
+
 static void
 test_communicate_utf8_invalid (void)
 {
@@ -1175,6 +1203,40 @@ test_pass_fd (void)
 
 #endif
 
+static void
+test_launcher_environment (void)
+{
+  GSubprocessLauncher *launcher;
+  GError *error = NULL;
+  GSubprocess *proc;
+  GPtrArray *args;
+  gchar *out;
+
+  g_setenv ("A", "B", TRUE);
+  g_setenv ("C", "D", TRUE);
+
+  launcher = g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDOUT_PIPE);
+
+  /* unset a variable */
+  g_subprocess_launcher_unsetenv (launcher, "A");
+
+  /* and set a diffferent one */
+  g_subprocess_launcher_setenv (launcher, "E", "F", TRUE);
+
+  args = get_test_subprocess_args ("printenv", "A", "C", "E", NULL);
+  proc = g_subprocess_launcher_spawnv (launcher, (const gchar **) args->pdata, &error);
+  g_assert_no_error (error);
+  g_assert (proc);
+
+  g_subprocess_communicate_utf8 (proc, NULL, NULL, &out, NULL, &error);
+  g_assert_no_error (error);
+
+  g_assert_cmpstr (out, ==, "C=D\nE=F\n");
+  g_free (out);
+
+  g_object_unref (proc);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -1201,6 +1263,7 @@ main (int argc, char **argv)
   g_test_add_func ("/gsubprocess/communicate-utf8", test_communicate_utf8);
   g_test_add_func ("/gsubprocess/communicate-utf8-async", test_communicate_utf8_async);
   g_test_add_func ("/gsubprocess/communicate-utf8-invalid", test_communicate_utf8_invalid);
+  g_test_add_func ("/gsubprocess/communicate-nothing", test_communicate_nothing);
   g_test_add_func ("/gsubprocess/terminate", test_terminate);
   g_test_add_func ("/gsubprocess/env", test_env);
   g_test_add_func ("/gsubprocess/cwd", test_cwd);
@@ -1210,6 +1273,7 @@ main (int argc, char **argv)
   g_test_add_func ("/gsubprocess/child-setup", test_child_setup);
   g_test_add_func ("/gsubprocess/pass-fd", test_pass_fd);
 #endif
+  g_test_add_func ("/gsubprocess/launcher-environment", test_launcher_environment);
 
   return g_test_run ();
 }
